# Commit Context

This is the context structure for each commit in the templates

| Name            | Type                                          | Description                                                                                                                                                                                                                                                                                                                                                                  |
|-----------------|-----------------------------------------------|------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| sha             | `string`                                      | The full hex SHA of the commit.                                                                                                                                                                                                                                                                                                                                              |
| short_sha       | `string`                                      | The first seven characters of the hex sha.                                                                                                                                                                                                                                                                                                                                   |
| commit_datetime | `datetime`                                    | The date and time of the commit with timezone offset.                                                                                                                                                                                                                                                                                                                        |
| summary         | `string`                                      | The first line of the commit message.                                                                                                                                                                                                                                                                                                                                        |
| body            | `string`                                      | The commit message sans the first line.                                                                                                                                                                                                                                                                                                                                      |
| committer       | `string`                                      | The name and email of the committer as `name <email@ex.com>`.                                                                                                                                                                                                                                                                                                                |
| authors         | `list` of `dict` with `name` and `email` keys | A list of authors' names and emails related to the commit.<br><br>This will always contain the committer. If the [`ParseTrailers`][generate_changelog.actions.metadata.ParseTrailers] Action was configured in the {attr}`~.Configuration.body_pipeline` (the default), this will also include names and emails of people included in the commit trailers as configured via {attr}`~.Configuration.valid_author_tokens`. |
| author_names    | `list` of `string`                            | A list of just author names, extracted from `authors`.                                                                                                                                                                                                                                                                                                                       |
| grouping        | `tuple`                                       | The values to group this commit based on the {attr}`~.Configuration.group_by` configuration.                                                                                                                                                                                                                                                                                 |
| metadata        | `dict`                                        | Metadata for this commit parsed from the commit message. Metadata is generated from Actions in a Pipeline.                                                                                                                                                                                                                                                                   |

## Commit metadata

| Name                | Type               | Availability                                                                                                                                                                                                                                                                                                                                  | Description                                                                                                                    |
|---------------------|--------------------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------|
| category            | `string`           | <ul><li>[`SummaryRegexMatch`][generate_changelog.actions.matching.SummaryRegexMatch]</li><li>[`MetadataMatch`][generate_changelog.actions.matching.MetadataMatch]</li></ul>                                                                                                                                                                   | This value is set using the first Action in the {attr}`~.Configuration.commit_classifiers` configuration that returns a value. |
| trailers            | `dict`             | <ul><li>[`ParseTrailers`][generate_changelog.actions.metadata.ParseTrailers]</li></ul>                                                                                                                                                                                                                                                        | The git trailers parsed out of the commit body.                                                                                |
| issue               | `list` of `string` | <ul><li>[`ParseIssue`][generate_changelog.actions.metadata.ParseIssue]</li><li>[`ParseGitHubIssue`][generate_changelog.actions.metadata.ParseGitHubIssue]</li><li>[`ParseJiraIssue`][generate_changelog.actions.metadata.ParseJiraIssue]</li><li>[`ParseAzureBoardIssue`][generate_changelog.actions.metadata.ParseAzureBoardIssue]</li></ul> | A list of issues parsed from the commit.                                                                                       |
| has_breaking_change | `boolean`          | <ul><li>[`ParseBreakingChangeFooter`][generate_changelog.actions.metadata.ParseBreakingChangeFooter]</li><li>[`ParseConventionalCommit`][generate_changelog.actions.metadata.ParseConventionalCommit]</li></ul>                                                                                                                               | Indicates this commit has a breaking change. It doesn't mean that there is a description of the breaking change.               |
| breaking_changes    | `string`           | <ul><li>[`ParseBreakingChangeFooter`][generate_changelog.actions.metadata.ParseBreakingChangeFooter]</li></ul>                                                                                                                                                                                                                                | Description of breaking changes as parsed from the commit body.                                                                |
| scope               | `list` of `string` | <ul><li>[`ParseConventionalCommit`][generate_changelog.actions.metadata.ParseConventionalCommit]</li></ul>                                                                                                                                                                                                                                    | Zero or more scopes parsed from the commit summary.                                                                            |
